/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Huawei Technologies
 *
 */

#ifndef __POSIX_API_H__
#define __POSIX_API_H__

#include <signal.h>
#include <sys/poll.h>
#include <sys/epoll.h>
#include <sys/eventfd.h>

typedef struct {
    void *handle;
    int (*socket_fn)(int domain, int type, int protocol);
    int (*accept_fn)(int s, struct sockaddr*, socklen_t*);
    int (*accept4_fn)(int s, struct sockaddr *addr, socklen_t *addrlen, int flags);
    int (*bind_fn)(int s, const struct sockaddr*, socklen_t);
    int (*listen_fn)(int s, int backlog);
    int (*connect_fn)(int s, const struct sockaddr *name, socklen_t namelen);
    int (*getpeername_fn)(int s, struct sockaddr *name, socklen_t *namelen);
    int (*getsockname_fn)(int s, struct sockaddr *name, socklen_t *namelen);
    int (*setsockopt_fn)(int s, int level, int optname, const void *optval, socklen_t optlen);
    int (*getsockopt_fn)(int s, int level, int optname, void *optval, socklen_t *optlen);
    int (*shutdown_fn)(int s, int how);
    int (*close_fn)(int fd);
    pid_t (*fork_fn)(void);
    ssize_t (*read_fn)(int fd, void *mem, size_t len);
    ssize_t (*write_fn)(int fd, const void *data, size_t len);
    ssize_t (*recv_fn)(int sockfd, void *buf, size_t len, int flags);
    ssize_t (*send_fn)(int sockfd, const void *buf, size_t len, int flags);
    ssize_t (*recv_msg)(int sockfd, const struct msghdr *msg, int flags);
    ssize_t (*send_msg)(int sockfd, const struct msghdr *msg, int flags);
    ssize_t (*recv_from)(int sockfd, void *buf, size_t len, int flags, struct sockaddr *src_addr, socklen_t *addrlen);
    ssize_t (*send_to)(int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, 
        socklen_t addrlen);
    int (*fcntl_fn)(int fd, int cmd, ...);
    int (*fcntl64_fn)(int fd, int cmd, ...);
    int (*pipe_fn)(int pipefd[2]);
    int (*epoll_create_fn)(int size);
    int (*epoll_ctl_fn)(int epfd, int op, int fd, struct epoll_event *event);
    int (*epoll_wait_fn)(int epfd, struct epoll_event *events, int maxevents, int timeout);
    int (*epoll_close_fn)(int epfd);
    int (*eventfd_fn)(unsigned int initval, int flags);
    int (*is_epfd)(int fd);
    struct lwip_sock* (*get_socket)(int fd);
    int (*sigaction_fn)(int signum, const struct sigaction *act, struct sigaction *oldact);
    int (*poll_fn)(struct pollfd *fds, nfds_t nfds, int timeout);
    int (*ioctl_fn)(int fd, int cmd, ...);

    int ues_posix;
} posix_api_t;

extern posix_api_t *posix_api;

int posix_api_init(void);
void posix_api_free(void);
void posix_api_fork(void);

#endif /* __POSIX_API_H__ */
